# Distribution Checker
# Moblin Clutter Test Module (Perl_test.pm)
#
# Copyright (C) 2007-2009 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
# The ptyshell tool is originally written by Jiri Dluhos <jdluhos@suse.cz>
# Copyright (C) 2005-2007 SuSE Linux Products GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

package Moblin_clutter_test;
use strict;

use Misc;
use Subshell;

use Test_common;
our @ISA = qw(Test_common); # Inherit Test_common
#----------------------------------------------------------------------

my $checkpoints = undef;

#----------------------------------------------------------------------

sub prepare {
	my ($self) = @_;

	# Call the parent's prepare function
	is_ok $self->Test_common::prepare() or return $Error::Last;

	cmd ("killall -q Xvfb"); # Should be done before calling guess_display()
	sleep 1;

	my $testsuite_dir = $self->option('TESTSUITE_DIR') || "/opt/moblin/test/moblin-test/moblin-test-clutter/tests/auto";
	$self->{TESTSUITE_DIR} = $testsuite_dir;
	$self->{RESULTS_DIR} = $self->option('RESULTS_DIR') || "/opt/moblin/test/moblin-test/moblin-test-clutter/tests/auto/result";

	init_checkpoint_data($self->{VERSION});

	# Remove old results
	cmd("rm -rf ".shq($self->{RESULTS_DIR})."/*");
	
	return 1;
}

sub run {
	my ($self) = @_;
	
	my $subshell = $self->Spawn_subshell();
	is_ok $subshell or return $Error::Last;
	
	# Change to the test suite directory
	$subshell->Settle();
	$subshell->Send( "cd ".shq($self->{TESTSUITE_DIR})."\n" );
	
	# Run the test
	$subshell->Settle();
	$subshell->Send("./clutter_autotest.sh; exit\n");

	my $started = 0;
	
	# Just wait for the test finished
	while ( my $line = $subshell->Read(2) ) {
		if ( $line =~ /Starting X Virtual Frame Buffer/ ) {
			$started = 1;
		}
		
		$self->check_progress($line);
	}
	
	# Just wait for the test finished
	$subshell->Settle();
	$subshell->WaitForSubshell();
	
	if ( $subshell->{EXIT_CODE} ) {
		$self->report_warning(warning "subshell exited with code ".$subshell->{EXIT_CODE}
			.".\n".$subshell->get_last_lines);
	}

	if ( !$started ) {
		return error "The test hasn't started";
	}

	return 1; # Ok
}

sub check_progress {
	my ($self, $line) = @_;
	
	defined $line or $line = "";
	
	if ( $line =~ /^\e\[33m(.+\S)\s+\e.*[\r\n]/ ) {
		my $testcase = $1;
		$self->progress_profiled($checkpoints, $testcase);
		return;
	}
	# else:
	$self->progress_profiled();
}

sub after_run {
	my ($self) = @_;

	$self->{JOURNAL_FILE} = $self->name.".journal";
	$self->{JOURNAL_TYPE} = 'TET';
	
	# Find the journal file and copy it to where it should be.
	is_ok $self->copy_file_globbed( $self->{RESULTS_DIR}."/journal.all", $self->journal_file )
		or $self->report_error(complain $Error::Last);
	
	return 1;
}

sub Cleanup {
	my ($self) = @_;
	
	cmd ("killall -q Xvfb");
}

sub init_checkpoint_data {
	my ($ver) = @_;
	# moblin-clutter v. 0.6.2-1
	$checkpoints = [
		[ 35, '/meta/getcontainer_NullMeta' ],
		[ 42, '/clutter_cairo_texture/create_normal' ],
		[ 49, '/actor/lowerbtm' ],
		[ 56, '/stage/pers_setNullStage' ],
		[ 63, '/score/append_parent_not_in_score' ],
		[ 70, '/timeline/advance_to_marker_NULL_timeline' ],
		[ 77, '/timeline/skip_beyond_duration' ],
		[ 84, '/behaviour/get_nth_actor_NULL_behave' ],
		[ 91, '/behaviour/ellipse_set_angle_end_NULL' ],
		[ 98, '/behaviour/ellipse_set_angle_start_angle_more_than_360' ],
	];
}

1; # return value
